//go:build example
// +build example

package game

import (
	"image/color"
	"os"
	"sync"

	"code.rocketnine.space/tslocum/gohan/examples/twinstick/world"

	"code.rocketnine.space/tslocum/gohan"
	"code.rocketnine.space/tslocum/gohan/examples/twinstick/asset"
	"code.rocketnine.space/tslocum/gohan/examples/twinstick/entity"
	"code.rocketnine.space/tslocum/gohan/examples/twinstick/system"
	"github.com/hajimehoshi/ebiten/v2"
)

// game is an isometric demo game.
type game struct {
	w, h int

	player gohan.Entity

	op *ebiten.DrawImageOptions

	disableEsc bool

	debugMode  bool
	cpuProfile *os.File

	sync.Mutex
}

// NewGame returns a new isometric demo game.
func NewGame() (*game, error) {
	g := &game{
		op: &ebiten.DrawImageOptions{},
	}

	world.Player = entity.NewPlayer()

	g.addSystems()

	err := g.loadAssets()
	if err != nil {
		return nil, err
	}

	asset.ImgWhiteSquare.Fill(color.White)

	gohan.Preallocate(10000)

	return g, nil
}

// Layout is called when the game's layout changes.
func (g *game) Layout(outsideWidth, outsideHeight int) (int, int) {
	s := ebiten.DeviceScaleFactor()
	w, h := int(s*float64(outsideWidth)), int(s*float64(outsideHeight))
	if w != g.w || h != g.h {
		g.w, g.h = w, h
		world.ScreenW, world.ScreenH = float64(w), float64(h)
	}
	return g.w, g.h
}

func (g *game) Update() error {
	if ebiten.IsWindowBeingClosed() {
		g.Exit()
		return nil
	}

	return gohan.Update()
}

func (g *game) Draw(screen *ebiten.Image) {
	err := gohan.Draw(screen)
	if err != nil {
		panic(err)
	}
}

func (g *game) addSystems() {
	// Handle input.
	gohan.AddSystem(system.NewProfileSystem())
	gohan.AddSystem(system.NewMovementInputSystem())
	gohan.AddSystem(system.NewMovementSystem())
	gohan.AddSystem(system.NewFireInputSystem())

	// Render game.
	gohan.AddSystem(system.NewDrawBulletsSystem())
	gohan.AddSystem(system.NewDrawPlayerSystem())
	gohan.AddSystem(system.NewPrintInfoSystem())
}

func (g *game) loadAssets() error {
	return nil
}

func (g *game) Exit() {
	os.Exit(0)
}
